import gzip
import json
import os
import threading

"""
本来使用神器shelve，但是实际使用下发现会丢失数据，可能是用得不对，自己封装一个简单的，够用了
"""


class Keep:
    def __init__(self, file_name='keep.data', auto_save=True, encoding="utf-16"):
        self.file_name = file_name
        self.d = {}
        self.lock = threading.RLock()
        self.auto_save = auto_save
        self.encoding = encoding
        self.f = None  # type:gzip.GzipFile
        if os.path.exists(file_name):
            with gzip.GzipFile(self.file_name, mode="rb") as f:
                s = f.read()
                if len(s) > 1:
                    self.d = json.loads(str(s, encoding=self.encoding))

    def open(self):
        """打开数据文件"""
        if self.f is not None and not self.f.closed:
            return
        else:
            self.f = gzip.GzipFile(self.file_name, mode="wb+")

    def save(self):
        """保存数据到数据文件"""
        self.lock.acquire()
        try:
            self.open()
            self.f.write(json.dumps(self.d).encode(self.encoding))
            self.f.flush()
        finally:
            self.lock.release()

    def get(self, key, default=None):
        """根据key获取value"""
        return self.d.get(key, default)

    def set(self, key, value):
        """设置值"""
        self.d[key] = value
        if self.auto_save:
            self.save()
            self.close()

    def delete(self, key):
        """删除值"""
        del self.d[key]
        if self.auto_save:
            self.save()
            self.close()

    def close(self):
        """关闭数据文件"""
        if self.f is None or self.f.closed:
            return
        self.f.close()


if __name__ == '__main__':
    k = Keep()
    k.set("AAA", "121212")
    print(k.get("AAA"))



def get_dict(entity):
    dd = dir(entity)
    _dict = {}
    for d in dd:
        if not d.startswith("__"):
            _dict[d] = getattr(entity, d)
    return _dict


def get_insert_sql(entity, fn_db_value):
    dd = get_dict(entity)
    VALUES = []
    for d in dd:
        value = getattr(entity, d)
        if value is None:
            VALUES.append("null")
        elif not hasattr(entity, "__columns") or getattr(entity, "__columns") is None:
            VALUES.append(f"'{value}'")
        else:
            data_type = getattr(entity, "__columns").get(d)
            value = fn_db_value(data_type=data_type, value=value)
            VALUES.append(value)

    return f"""INSERT INTO {getattr(entity, "__table_name")}({",".join(dd)}) VALUES ({",".join(VALUES)})"""


def get_update_sql(entity, fn_db_value):
    keys = getattr(entity, "__primary_key")
    where = "WHERE"
    ps = []
    for k in keys:
        data_type = getattr(entity, "__columns").get(k)
        value = fn_db_value(data_type=data_type, value=getattr(entity, k))
        ps.append(f" {k}={value}")
    where += ' AND '.join(ps)

    dd = get_dict(entity)
    VALUES = []
    for d in dd:
        if d in keys:
            continue
        value = getattr(entity, d)
        if value is None:
            VALUES.append("null")
        elif not hasattr(entity, "__columns") or getattr(entity, "__columns") is None:
            VALUES.append(f"'{value}'")
        else:
            data_type = getattr(entity, "__columns").get(d)
            value = fn_db_value(data_type=data_type, value=value)
            VALUES.append(f"{d}={value}")

    return f"""UPDATE {getattr(entity, "__table_name")} SET {",".join(VALUES)} {where}"""



from db_hammer.base import BaseConnection
from db_hammer.csv import start as csv_start

try:
    import pymysql
except ImportError:
    print("cant import pymysql")
    print("===> pip3 install pymysql")
    raise Exception("import pymysql error")


class MySQLConnection(BaseConnection):
    def __init__(self, **kwargs):
        if kwargs.get("host", None) is None:
            raise Exception("host")
        if kwargs.get("user", None) is None:
            raise Exception("user")
        if kwargs.get("db_name", None) is None:
            raise Exception("db_name")
        if kwargs.get("pwd", None) is None:
            raise Exception("pwd")
        port = kwargs.get("port", 3306)
        charset = kwargs.get("charset", "utf8")
        super().__init__(**kwargs)
        self.conn = pymysql.connect(kwargs["host"], kwargs["user"], kwargs["pwd"], kwargs["db_name"], port=port,
                                    charset=charset)
        self.cursor = self.conn.cursor()

    def convert_str(self, s: str):
        return s.replace("'", "\\'")

    def export_data_file(self, sql, dir_path, file_mode="gz", pack_size=500000, bachSize=10000, add_header=True,
                         data_split_chars=',',
                         data_close_chars='"', encoding="utf-8", outingCallback=None):
        """导出数据文件
        @:param sql 导出时的查询SQL
        @:param dir_path 导出的数据文件存放目录
        @:param file_mode 导出文件格式：txt|gz|csv
        @:param add_header 数据文件是否增加表头
        @:param pack_size  每个数据文件大小，默认为50万行，强烈建议分割数据文件，单文件写入速度会越来越慢
        @:param bachSize   游标大小
        @:param data_split_chars 每条数据字段分隔字符,csv文件默认为英文逗号
        @:param data_close_chars 每条数据字段关闭字符,csv文件默认为英文双引号
        @:param encoding 文件编码格式，默认为utf-8
        @:param outingCallback 导出过程中的回调方法
        """
        # 要使用服务器游标，本地游标会内存溢出
        cursor = self.conn.cursor(pymysql.cursors.SSCursor)
        csv_start(cursor=cursor,
                  sql=sql,
                  path=dir_path,
                  bachSize=bachSize,
                  PACK_SIZE=pack_size,
                  file_mode=file_mode,
                  add_header=add_header,
                  CSV_SPLIT=data_split_chars,
                  CSV_FIELD_CLOSE=data_close_chars,
                  encoding=encoding,
                  callback=outingCallback,
                  log=self.log)
        cursor.close()



import logging
import os
import sys
from pathlib import Path

LOG_MAP = {}
LOGGING_FORMATTER = '%(asctime)s - %(processName)s:%(threadName)s - %(filename)s[line:%(lineno)d] - %(levelname)s: %(message)s'


def getLogging(name, path="", level=logging.DEBUG) -> logging:
    if path == "":
        path = str(Path.home())

    logger = logging.getLogger(name)
    if LOG_MAP.get(name, None) is None:
        LOG_MAP[name] = name
        logger.setLevel(level)  # Log等级总开关
        formatter = logging.Formatter(LOGGING_FORMATTER)
        ch = logging.StreamHandler(sys.stdout)
        ch.setLevel(logging.DEBUG)  # 输出到console的log等级的开关
        ch.setFormatter(formatter)
        logger.addHandler(ch)
        os.makedirs(f"{path}/logs", exist_ok=True)
        logfile = f'{path}/logs/{name}.log'
        fh = logging.FileHandler(logfile)
        fh.setLevel(logging.DEBUG)
        fh.setFormatter(formatter)
        logger.addHandler(fh)
    return logger



import datetime


def date_to_str(date=None, format_str="%Y-%m-%d %H:%M:%S"):
    """时期格式化成字符
        :param date 时间
        :param format_str %Y-%m-%d %H:%M:%S
    """
    if date is None:
        date = datetime.datetime.now()
    return date.strftime(format_str)


def get_current():
    return datetime.datetime.now()


def get_date_str(date=None, format_str="%Y-%m-%d"):
    if date is None:
        date = datetime.datetime.now()
    return date.strftime(format_str)



from enum import Enum


def table(name=None):
    """数据据保存的表名"""

    def decorate(clazz):
        if name is None:
            setattr(clazz, "__table_name", clazz.__name__)
        else:
            setattr(clazz, "__table_name", name)

        return clazz

    return decorate


def primary_key(keys):
    """配置主键"""
    if isinstance(keys, str):
        keys = str(keys).split(",")

    def decorate(clazz):
        setattr(clazz, "__primary_key", keys)
        return clazz

    return decorate


def columns(**kwargs):
    """数据库保存时列名定义"""
    def decorate(clazz):
        __columns = {}
        for k in kwargs:
            __columns[k] = kwargs[k]
        setattr(clazz, "__columns", __columns)
        return clazz
    return decorate



from setuptools import setup

with open("./README.md", "rb") as fh:
    long_description = fh.read()

setup(
    name='db-hammer',
    version='0.0.12',
    description='database tools；数据库操作',
    author='hammer',
    author_email='liuzhuogood@foxmail.com',
    long_description=str(long_description, encoding='utf-8'),
    long_description_content_type="text/markdown",
    packages=['db_hammer', 'db_hammer.util'],
    package_data={'db_hammer': ['README.md', 'LICENSE']},
    install_requires=[]
)



import subprocess


def execute(cmd):
    p = subprocess.Popen(cmd,
                         shell=True,
                         stdin=subprocess.PIPE,
                         stdout=subprocess.PIPE,
                         stderr=subprocess.PIPE)
    stdout, stderr = p.communicate()
    if p.returncode != 0:
        return p.returncode, stderr
    return p.returncode, stdout


def execute_get_code(cmd):
    return execute(cmd)[0]


def execute_get_sout(cmd):
    return str(execute(cmd)[1], 'utf-8')


if __name__ == '__main__':
    print(execute_get_sout("ls"))
